@@ -502,6 +502,7 @@ io_watch_stdout (GIOChannel *source, GIOCondition condition, PasswordDialog *pdi
 
 			if (is_string_complete (str->str, "successfully",
 											  "short",
+											  "longer",
 											  "panlindrome",
 											  "simple",
 											  "similar",
@@ -531,17 +532,19 @@ io_watch_stdout (GIOChannel *source, GIOCondition condition, PasswordDialog *pdi
 						/* What does this indicate?
 						 * "Authentication information cannot be recovered?" from libpam? */
 						msg = g_strdup_printf (_("System error: %s."), str->str);
-					} else if (g_strrstr (str->str, "short") != NULL) {
+					} else if (g_strrstr (str->str, "short") != NULL ||
+						   g_strrstr (str->str, "longer") != NULL) {
 						msg = g_strdup (_("The password is too short."));
-					} else if (g_strrstr (str->str, "panlindrome") != NULL) {
+					} else if (g_strrstr (str->str, "panlindrome") != NULL ||
+						   g_strrstr (str->str, "simple") != NULL) {
 						msg = g_strdup (_("The password is too simple."));
-					} else if (g_strrstr (str->str, "simple") != NULL) {
-						msg = g_strdup (_("The password is too simple."));
-					} else if ((g_strrstr (str->str, "similar") != NULL) || (g_strrstr (str->str, "wrapped") != NULL)) {
+					} else if (g_strrstr (str->str, "similar") != NULL ||
+						   g_strrstr (str->str, "wrapped") != NULL) {
 						msg = g_strdup (_("The old and new passwords are too similar."));
 					} else if (g_strrstr (str->str, "1 numeric or special") != NULL) {
 						msg = g_strdup (_("The new password must contain numeric or special character(s)."));
-					} else if ((g_strrstr (str->str, "unchanged") != NULL) || (g_strrstr (str->str, "match") != NULL)) {
+					} else if (g_strrstr (str->str, "unchanged") != NULL ||
+						   g_strrstr (str->str, "match") != NULL) {
 						msg = g_strdup (_("The old and new passwords are the same."));
 					} else if (g_strrstr (str->str, "failure") != NULL) {
 						/* Authentication failure */
